<?php
/**
 * $Id$ 
 * 
 * Drupal carbon account module 
 *
 * Created on 18-Oct-2006
 * 
 * @file 
 * definition of carbon_account node. Each carbon_account
 * node contains settings such as choice of preferred carbon 
 * model and privacy settings that control the presentation of 
 * a carbon footprint.   
 */



/**
 * Implementation of hook_access().
 * 
 * Refer to carbon_access because it is easier to check 
 * access for all carbon types in one place.  
 */


function carbon_account_access($op, $node)
{
  return carbon_access($op, $node);
}


/**
 * Return an array of protection options
 * @return array
 */
function _get_protection_options() {
  return array(t("private"), t("share with others in my group"), 
               t("share with all logged in users"), t("public"));
}


/**
 * Implementation of hook_form().
 */


function carbon_account_form(&$node, &$param) {
  global $user;
  $source = carbon_source_fetch_array();    
    
  $model_map = _get_model_map($source);
  $source_map = _get_source_map($source);
  $source_keys = array_keys($source_map); 

  $form = array();  
  $form = carbon_account_account_form($form, $node, $model_map); 

  if (variable_get("carbon_in_situ_edit", 0)== 0)
    return $form ;
    
// OK we are doing in situ stamp editing   
  
  $form["#prefix"] = "<p>".t("You can make basic and quick changes to your footprint below. " .
    " When you click on submit, new carbon stamps will be added and existing " .
    " carbon stamps will be updated in one go. Each carbon stamp represents " .
    " a source or emissions. </p><p>" .      
    " To finely tune your footprint, enter " .
    " carbon stamps separately. This can be done by clicking on the " .
    " <i>view</i> tab above and then <i>edit</i>, <i>clone</i> or <i>add new</i> " .
    " carbon stamps. ")."</p>";  
  
// if this is a new node, don't accept stamps yet

// if (!isset($node->nid) || $node->nid == -1)
//   return $form ;    

  $i = $group = 0 ;       
  $processed = array() ;  

  if (!isset($node->stamp))
    $node->stamp = array(); 
    
  foreach ($node->stamp as $stamp ) 
  {                
    foreach($source_keys as $key)
    {     
        // we have identified on of the utilities
        // so now go through the list again
            
      $grouphash = _concat($stamp);
             
      if (!array_key_exists($grouphash, $processed))
      { 
      // we have identified a group created by the user 
      // that has not been displayed so create a set 
      // of forms for EACH source in the group/
            
        $add_new_field = false; 
        foreach ($node->stamp as $stamp3)
        { 
          if ($grouphash == _concat($stamp3))
          {                             
             _stamp_repeating_form($form, $stamp3, $stamp3, $i, $group); 
             $i++  ;                             
             $add_new_field = true ; 
          }                                       
        }
        if ($add_new_field)
        {
          _stamp_repeating_form($form, null, $stamp, $i, $group); 
          $i++ ;
        }
        $group++ ;
        $processed[$grouphash] = 0 ;
      }
    }             
  }
  $existing_codes = array();
  foreach ($node->stamp as $stamp4)
    $existing_codes[] = $stamp4->code ;


  $templates = carbon_account_template(); 
  foreach ($templates as $template)
  {
    if (_search_preg($template["code"], $existing_codes))
      continue ;    // we already got some of these so dont prompt
      
    // check that code not already being used, in which case 
    // the user has picked up the ball and is running 
    
    _stamp_form($form, $source_keys, array_values($source_map), null, $i, $group, $template["title"]);    
    _stamp_customize_form($form, $source_keys, array_values($source_map), null, $i, $group, $template); 
    $i++ ; 
    $group++ ;
  }
// for the benefit of advanced users
  _stamp_form($form, $source_keys, array_values($source_map), null, $i, $group, "My other source of emissions");
  $stamp_form = "f".$i ; 
  $stamp_group_form = "g".$group ;
  $form[$stamp_group_form]["#collapsed"] = true ;     
  $i++ ; $group++ ;
  
  return $form ;  
} 

/**
 * search an array of strings $target with 
 * another array of patterns and look for 
 * a match. 
 * 
 * @return true if match
 */
  
function _search_preg($pattern, $subject)
{
  foreach ($subject as $t)
  {
    foreach($pattern as $p)
    {
      if (preg_match("/".$p."/", $t))
        return true ; 
    }
  }  
  return false;     
}



/**
 * This template defines the default categories that users
 * might want to use to categorise their different sources 
 * of emissions.
 */

function carbon_account_template()
{
  return array(
  
    array(
        "title" => "My home gas",         
        "scope" => array(0), 
//      "weight" => 1,
        "code" => array("gas-m3", "gas-.*"), 
        "description" => "amount of gas used in the last year in cubic metres."),
        
    array(
        "title" => "My home electricity", 
        "scope" => array(0), 
//      "weight" => 2,
        "code" => array("elec-kwh","elec-.*"),
        "description" => "amount of electricity used in the last year in Kilowatt Hours."),   
         
    array(
        "title" => "My trip to college or work", 
        "scope" => array(2), 
//      "weight" => 3,   
        "code" => array("car-med-miles", "car-.*", "pt-.*"),
        "description" => "distance travelled by car to work each day in miles."),
        
    array("title" => "My air miles", 
        "scope" => array(0),
//      "weight" => 4,   
        "code" => array("air-miles", "air-.*"),
        "description" => "total air miles flown in the last year.")
    );  
}

/**
 * So as not to horify initial users with a lot of fields we present 
 * a small number of fields for users to fill in. 
 */

function _stamp_customize_form(&$form, $source_keys, $source_descriptions, $stamp, $i, $group, $template) 
{
  $stamp_group_form = "g".$group ;
  $stamp_form = "f".$i ; 
   
  $code_index = array_search($template["code"][0],$source_keys);
  
  $form[$stamp_group_form][$stamp_form]["ff".$i."title"]["#default_value"]= $template["title"] ;
    
  $form[$stamp_group_form][$stamp_form]["ff".$i."code_index"]["#type"] = "hidden" ;
  $form[$stamp_group_form][$stamp_form]["ff".$i."code_index"]["#value"] = $code_index ; 
  
  $form[$stamp_group_form][$stamp_form]["ff".$i."scope_index"]["#type"] = "hidden" ;
  $form[$stamp_group_form][$stamp_form]["ff".$i."scope_index"]["#value"] = $template["scope"][0]; 

  unset($form[$stamp_group_form][$stamp_form]["ff".$i."title"]["#description"]);    
  unset($form[$stamp_group_form][$stamp_form]["ff".$i."startdate_s"]["#type"]);
  unset($form[$stamp_group_form][$stamp_form]["ff".$i."enddate_s"]["#type"]);  
  unset($form[$stamp_group_form][$stamp_form]["ff".$i."adjustment"]["#type"]);
    
  $form[$stamp_group_form][$stamp_form]["form".$i."reading"]["#description"] = $template["description"];

  $form[$stamp_group_form]["#collapsed"] = true ; 
  $form[$stamp_group_form]["#weight"] = $template["weight"]  ; 
} 



function carbon_account_edit_1($nid)
{
    $empty = array(); 
    $node = new Node();
    $node->nid = $nid ; 
    $node = node_load($nid);  
    $form = carbon_account_form($node, $empty);
    
    $output .= drupal_get_form('carbon_stamp_table_edit',null);  //poss broken
    
    return $output ; 
}


/**
 * Return an array of task filing_statuses
 * @return array of filing_statuses
 */
function _filing_statuses() {
  return array("draft", "filed", "audited");
}


function carbon_account_account_form(&$form, $node, $model_map)
{ 
  $form["account"] = array(
    "#type" => "fieldset", 
    "#title" => "My preferences",
    "#weight" => -1,
//      '#prefix' => '<div class="container-inline">',
//      '#suffix' => '</div>',  
    "#collapsible" => TRUE,
    "#collapsed" => TRUE);

  $form["account"]["title"] = array(
    "#type" => "textfield", 
    "#title" => t("Title of this footprint"), 
    "#required" => TRUE, 
    "#default_value" => empty($node->title) ? $node->name : $node->title,
    "#size" => 20,
    "#maxlength" => 40, 
    "#weight" => -1,
    "#description" => t("Usually your name or someone else's name and can be different from your username."));
    
  if (variable_get("carbon_use_postal_code", 1))
  
    $form["account"]["postcode"] = array(
    "#type" => "textfield", 
    "#title" => t("Postcode"), 
    "#required" => FALSE, 
    "#default_value" => empty($node->postcode) ? '' : $node->postcode,  
    "#size" => 10,
    "#maxlength" => 20, 
    "#weight" => -1,
    "#description" => t("Either some or all of your postcode or zipcode. Interesting to others " .
            "if you share your footprint."));
            
  if (variable_get("carbon_use_organization", 1) 
  ||  user_access("administer accounts")
  ||  user_access("administer carbon nodes"))
   
    $form["account"]["organization"] = array(
    "#type" => "textfield",
    "#title" => t("Local Group"),
    "#default_value" => empty($node->organization) ? '' : $node->organization,
    "#size" => 20,
    "#maxlength" => 20, 
    "#description" => t("Name of any local group, leave blank if none."),
    "#required" => FALSE);

  $model_keys = array_keys($model_map); 
  if (variable_get("carbon_use_model_selection", 1))
  
    $form["account"]["model_index"] = array(
    "#type" => "select", 
    "#title" => t("Preferred emissions model"), 
    "#required" => FALSE, 
    "#options" => array_values($model_map), 
    "#default_value" => isset($node->model) ? array_search($node->model, $model_keys) : 0 ,
    "#description" => "Select the preferred model that you want to use to calculate the CO2 emissions." .
  " Not all the models support all possible <a href='/?q=carbon/source'>carbon " .
    " sources</a>; in which case substitutes will be made. Leave as -no preference- if you are not sure.",
    "#weight" => 2);
  $form["account"]["protection"] = array(
    "#type" => "select", 
    "#title" => t("Privacy"), 
    "#required" => FALSE, 
    "#default_value" => !isset($node->protection) ? 2 : $node->protection,
    "#description" => t("Who can see this carbon footprint?"),
    "#options" => _get_protection_options(),
    "#weight" => 2);


  $form["account"]["firstdate_s"] = array(
    "#type" => "textfield", 
    "#title" => t("First day of reporting period"),  
    "#required" => FALSE, 
    "#size" => 20,
    "#maxlength" => 20, 
    "#default_value" => empty($node->firstdate)  ? "" : _format_date($node->firstdate, "custom", "Y-m-d"),
    "#weight" => 3,
    "#description" => 
               t("Format: %time. Can be left blank. The latest 12 months of " .
                "carbon stamps are used to calculate your footprint which will be " .
                "displayed alongside others in charts et cetera (assuming your privacy " .
                "settings allow this). You can override that date " .
                "by setting this field to the end of the period you want to use " .
                "and you can change the length of the period below. ", 
                array("%time" => _format_date(time(), "custom", "Y-m-d"))));
        
  $form["account"]["period"] = array(
    "#type" => "textfield", 
    "#title" => t("Reporting period in months"),  
    "#required" => FALSE, 
    "#size" => 2,
    "#maxlength" => 20, 
    "#default_value" => empty($node->period) ? 12 : $node->period,
    "#weight" => 3,
    "#description" => 
                t("The length of the reporting period in months."));

  $form["account"]["enablekwh0"] = array(
    "#type" => "checkbox", 
    "#title" => t("Display renewable energy (KillowattHours) in results"), 
    "#required" => false, 
    "#default_value" => isset($node->enablekwh0)? $node->enablekwh0 : false,
    "#weight" => 3);
    
  $form["account"]["enablekwh1"] = array(
    "#type" => "checkbox", 
    "#title" => t("Display non-renewable energy (KillowattHours) in results"), 
    "#required" => false, 
    "#default_value" => isset($node->enablekwh1)? $node->enablekwh1 : false,
    "#weight" => 3);

  $form["account"]["enableco2"] = array(
    "#type" => "checkbox", 
    "#title" => t("Display CO2 emissions in results"), 
    "#required" => false, 
    "#default_value" => isset($node->enableco2)? $node->enableco2 : true,
    "#weight" => 3);

     
  return $form;
}

/**
 * Validate our forms
 * @param string form id
 * @param array form values
 */
function carbon_account_validate($form_id, $form_values) 
{
  carbon_stamp_validate($form_id, $form_values); 
}




/**
 * Implementation of hook_view, add our node specific information
 * @param node object to display
 * @param boolean is this a teaser or full node?
 * @param boolean is this displaying on its own page
 */
function carbon_account_view(&$node, $teaser = FALSE, $page = FALSE) 
{
//  if (!user_access('view carbon footprints')) {
//    return true;
//  }

  $node = node_prepare($node, $teaser);
  $carbon_account_info = theme("carbon_account_table_view", $node);  // try item_list
  $node->content["footprint"] = array("#value" => $carbon_account_info);  
  $node->body .= $carbon_account_info;
  $node->teaser .= $carbon_account_info;
  return $node ; 
}



/**
 * Theme function to display additional node data
 * @param node to display
 * @return HTML string with additional node information
 */
 
 
// then create an array of column names


/**
 * Theme function to display additional node data
 * @param node to display
 * @return HTML string with additional node information
 */
 
 
// then create an array of column names

function theme_carbon_account_table_view($node)
{
  if (count($node->stamp) == 0)
    return carbon_calculate_empty_table($node);  
    
  $header = carbon_stamp_header();

  // move the stamps retrieved for this footprint into an array 

  $stamps = array();
  
  foreach ($node->stamp as $stamp)    // add the stamps
  {        
    $stamps[] = $stamp;
  }

  // find out the range of dates that we are dealing with
  
  $dateRange = _stamp_date_range($stamps);

 // work out where to place the boundary between years
 // use months and days from any user preference 
 // otherwise use Jan 1 in year of first stamp
  
  if (!empty($node->firstdate))       
  {  
    $y = _format_date($dateRange['first'],  'custom', 'Y') - 1 ;
    $m = _format_date($node->firstdate, 'custom', 'm');
    $d = _format_date($node->firstdate, 'custom', 'd'); 
  }
  else
  {
    $y = _format_date($dateRange['first'],  'custom', 'Y');
    $m = 0 ; 
    $d = 0 ;      
  }
  $reportStart = mktime(0, 0, 0, $m, $d, $y);   
          
  // if we are writing a chart, get the start date 
  // calculate the footprint and display the results
  
  if (carbon_chart_requested())
  { 
    $firstdate_s = carbon_chart_get_startdate();
    $firstdate = mktime(0, 0, 0, substr($firstdate_s,4,2), substr($firstdate_s,6,2), substr($firstdate_s,0,4));
    
    $lastdate = _add_months($firstdate, $node->period);
    $results = carbon_calculate_total($node, $stamps, $firstdate, $lastdate);
    
    $dataset = array(); 
    $dataset[$node->title] = array(
      "data" => $results->sector_co2,
      "comment" => _format_date_range($firstdate, $lastdate)) ; 
    carbon_chart_generate($dataset, "CO2 emissions");
    exit(0);
  }

  $y = _format_date($firstdate ,  'custom', 'Y');
//  drupal_set_message('first date '. $y, 'info');
  $output = "" ;

  // the purpose of the for loop is only to prevent 
  // any software bug from printing numerous reports
  
  for ($i = 0 ; $i < 10 ; $i++)
  {
    $firstdayinyear = _add_months($reportStart, $i * $node->period);
    $firstdayinnextyear = _add_months($reportStart, ($i + 1) * $node->period);
               
    // calculate emissions for this account
  
    $results = carbon_calculate_total($node, $stamps, $firstdayinyear, $firstdayinnextyear);         

    // now format them in a table      
    $output .= carbon_calculate_present_results($results);
    
    // and embed the graphics
    if ($results->grand_co2 > 0)  // used to supply a date here
        $output .= carbon_chart_embed(variable_get("carbon_default_chart", null), 
                                      _format_date($firstdayinyear, "custom", "Ymd"));
        
    // the condition will normally cause the for loop to be broken    
    if ($firstdayinnextyear> $dateRange['last']) 
      break ; 
  }   

  
//  $output .= theme('pager', NULL, 50, 0);

  $output .= '<p></p>';
  return $output;
}


function theme_carbon_account_page($form) 
{
	
  
}


/**
 * Implemenation of hook_load 
 * @param node object to load additional information for
 * @return object with carbon fields 
 */
function carbon_account_load($node)  {
  global $user ; 
  $query = db_query("SELECT f.*, s.uid as friends_uid " .
    "FROM {carbon_account} f " .
    "LEFT  JOIN {carbon_account_share} s ON  f.nid = s.nid " . 
    "WHERE f.nid = %d ",
    $node->nid);
    
  // most user will not be sharing records. However if they are 
  // we pick up their friends uid's while loading the footprint 
  $first_record = null ; 
  while ($record = db_fetch_object($query))
  {
    if (!isset($first_record))
      $first_record = $record ;   
      $first_record->friends = array(); 
      
    if (isset($friends_uid))
      $first_record->friends[] = $friends_uid ;       
  }
  $result = carbon_stamp_array_load($node->nid, 0, '');
  $first_record->stamp = $result['stamp'];
  $first_record->source = $result['source'];

//  drupal_set_message('loaded '.count($result['stamp']). ' stamps for '.$node->nid, 'info'); 

  return $first_record;
}



function carbon_account_submit(&$form)
{
  $sources = carbon_source_fetch_array() ;
  $source_map = _get_source_map($sources);
  $model_map = _get_model_map($sources);
  $model_keys = array_keys($model_map);

  if (!empty($form->firstdate_s))
  {
    $form->firstdate = strtotime($form->firstdate_s) ;
  }
  
  if (isset($form->model_index))
  {
    $form->model = $model_keys[$form->model_index] ;
  }
}



 
function carbon_account_handle_updates(&$node, $sources) 
{   
  global $user ; 
  $source_keys = array_keys(_get_source_map($sources));

//  drupal_set_message("number of sources is " .sizeof($source_keys) , 'info');

  // it's bad that we we have load these stamps again
  if (!isset($node->stamp))
  {
    $result = carbon_stamp_array_load($node->nid, 0, '');
    $node->stamp = $result["stamp"];
//    $node->source = $result['source'];    
  }
//  drupal_set_message('loaded '. count($source_keys). ' sources '.
//                count($node->stamp) . ' stamps ', 'info');

  for ($i=0 ; ; $i++)
  {
    $form = new Node ;
    _map_stamp_fields($form, $node, $i, $source_keys);

    // test for a combination of any fields that must 
    // be required to do an update
        
    if (empty($form->cloneid) && !isset($form->code_index))
      break ;   // probably no other fields to update. 

    if (empty($form->nid))
    {   
      if (!empty($form->reading))
      {
        // stamp inserts  
        $stamp = new Node;  
        if (isset($form->cloneid))
        {
          if (!isset($node->stamp[$form->cloneid]))
          {
            drupal_set_message("cannot clone, copy of existing stamp not available","info");
            continue ;
          }
          $template = $node->stamp[$form->cloneid] ; 
          $stamp->title = $template->title ; 
          $stamp->code = $template->code; 
          $stamp->scope = $template->scope; 
          $stamp->startdate = $template->startdate;           
          $stamp->adjustment = $template->adjustment; 
          $stamp->protection = $template->protection; 
          $stamp->exclude = $template->exclude;         
          $stamp->status = 1; 
        }
        else
        {
          $stamp->protection = 0 ; 
          $stamp->exclude = false ;   
          $stamp->adjustment =  1 ;       
        }
        // ask carbon_stamp_insert() (invoked by node_save) to attach to this footprint
        $stamp->fid_delete = array();
        $stamp->fid_insert = array($node->nid) ; 
        $stamp->uid = $node->uid ;      // inherit from footprint   
        $stamp->type = "carbon_stamp" ;       
        $stamp->status = 1 ;
        
        // now override all the values provided in the form
        
        _map_stamp_fields($stamp, $node, $i, $source_keys);
        node_save($stamp);
        drupal_set_message("Added ".$stamp->title.', '.$stamp->code.'='.$stamp->reading, "info"); 
      }
    } 
    else
    {
      // this is an existing stamp, we are just tweaking a few fields 
      // get the original node from the footprint.
      

      if (empty($node->stamp[$form->cloneid]))
      {
        drupal_set_message("cannot update, copy of existing stamp not available to compare","info");
        continue ;
      }

      $stamp = $node->stamp[$form->nid] ; 
      if ( $stamp->reading != $form->reading ||   
           $stamp->enddate != $form->enddate
         )
      {
        _map_stamp_fields($stamp, $node, $i, $source_keys);
        node_save($stamp);
        drupal_set_message("Updated ".$stamp->title.', '.$stamp->code.'='.$stamp->reading, "info");
      }   
    }     
  }
}


/**
 * Implementation of hook_insert, which saves carbon-specific information
 * into the carbon table
 * @param node object
 */

function carbon_account_insert(&$node) 
{   
  db_query("INSERT INTO {carbon_account} (postcode, organization, model, firstdate, period, " .
    " enablekwh0, kwh0, enablekwh1, kwh1, enableco2, co2, " .
    " filing_status, protection, nid) " .
    " VALUES ('%s', '%s', '%s', %d,  %d, %d, %d, %d, %d, %d, %d, %d, %d, %d)", 
  $node->postcode, $node->organization, $node->model, $node->firstdate, $node->period,
  $node->enablekwh0, $node->kwh0, $node->enablekwh1, $node->kwh1, $node->enableco2, $node->co2, 
  $node->filing_status, $node->protection,$node->nid);
  drupal_set_message(t("To retrieve your carbon account on a later " .
        "date click on <em>my account</em>."));
        
  $sources = carbon_source_fetch_array() ;
  carbon_account_handle_updates($node, $sources); 
}

/**
 * Implementation of hook_update, which saves updated todo-specific 
 * information into the todo table
 * 
 * We have to carefully step through the submitted form and compare
 * the values in the form with the values in the array 'ar''
 * @param node object
 */
function carbon_account_update(&$node) 
{     
  $result = db_query("UPDATE {carbon_account} " .
        " SET postcode='%s', organization='%s', model='%s', firstdate=%d, period=%d, " .
        " enablekwh0=%d, kwh0=%d, enablekwh1=%d, kwh1=%d, enableco2=%d, co2=%d, " . 
        " filing_status=%d, protection=%d WHERE nid=%d", 
  $node->postcode, $node->organization, $node->model, $node->firstdate, $node->period,
  $node->enablekwh0, $node->kwh0, $node->enablekwh1, $node->kwh1, $node->enableco2, $node->co2,
  $node->filing_status, $node->protection,$node->nid);

  $sources = carbon_source_fetch_array() ;
  carbon_account_handle_updates($node, $sources);
  return $result ;   
}


/**
 * Implementation of hook_delete().
 */
function carbon_account_delete($node) {
  db_query("DELETE FROM {carbon_stamp_account} WHERE fid = %d", $node->nid); 
  db_query("DELETE FROM {carbon_account_share} WHERE nid = %d", $node->nid); 
  db_query("DELETE FROM {carbon_account}       WHERE nid = %d", $node->nid);
}


/**
 * Add whole years onto a date
 * 
 * @param unix timestamp (secs from 1970)
 * @param integer number of months
 * @return the new date (secs from 1970)
 */

function _add_months($date, $num)
{
    return strtotime($num." month", $date);
}

/**
 * bypass Drupal's format_date() which deals only with offsets and 
 * appears to not understand daylight savings time.
 * 
 * @param $timestamp
 * @param $type
 * @return unknown_type
 */


function _format_date($timestamp, $type = 'medium', $format = '')
{
	// lifted formatting options from drupal code for consistency
  switch ($type) {
    case 'small':
      $format = variable_get('date_format_short', 'm/d/Y - H:i');
      break;
    case 'large':
      $format = variable_get('date_format_long', 'l, F j, Y - H:i');
      break;
    case 'custom':
      // No change to format
      break;
    case 'medium':
    default:
      $format = variable_get('date_format_medium', 'D, m/d/Y - H:i');
  }	
	return date($format, $timestamp);
}



/** 
 * Format a date range. The main purpose of this is to show a 
 * date range using as few as characters as possible so that 
 * it takes little space on the screen without being ambiguous.
 */

function _format_date_range($firstdate, $lastdate)
{
  $y1 = _format_date($firstdate, "custom", "y"); $y2 = _format_date($lastdate, "custom", "y");
  $m1 = _format_date($firstdate, "custom", "m"); $m2 = _format_date($lastdate, "custom", "m");

  if ($y1 == $y2)
  {
    if ($m1 == $m2)
      return _format_date($firstdate,"custom", "M Y");
    
    return _format_date($firstdate,"custom", "M")." to "._format_date($lastdate,"custom", "M Y");   
  }
  return _format_date($firstdate,"custom", "M Y")." to "._format_date($lastdate,"custom", "M Y");        
}



/**
 * walk through an array of carbon_stamps and find 
 * out the earliest and latest date for the stamps 
 * 
 * @return an array containing 'first' and 'last' dates.
 */


function _stamp_date_range($ar)
{
  $dateRange = array();
  $dateRange['first'] = $dateRange['last'] = null ;   
  foreach ($ar as $stamp)        
  {
    if (!empty($stamp->startdate))
    {  
      $dateRange['first'] = isset($dateRange['first']) ? 
        min($stamp->startdate, $dateRange['first']) : $stamp->startdate;
    }           
    if (!empty($stamp->enddate))
    {  
      $dateRange['first'] = isset($dateRange['first']) ? 
        min($stamp->enddate, $dateRange['first']) : $stamp->enddate;
      $dateRange['last'] = isset($dateRange['last']) ? 
        max($stamp->enddate, $dateRange['first']) : $stamp->enddate;
    }
  }
  return $dateRange;
}


/**
 * compress all the emission properties into a string as 
 * key value pairs and store in a single field 
 */ 

function _implode_data(&$node)
{
  $data = "" ; 
  $key = _data_type();
  
  foreach ($key as $entry) 
  {
    if (strlen($node->$entry) > 0)
      $data .= $entry.":".$node->$entry.";"; 
  }
  return $data ; 
}

/**
 * gather a range of parameters using GET
 * used by carbon_report()
 * 
 * @param $names is array of keys / default values
 * @return array of keys / values
 * */

function _gather($names)
{
  $q = array();
  foreach ($names as $name => $default_value)
  {
    if (!empty($_GET[$name]))
      $q[$name] = $_GET[$name] ;
    else 
      $q[$name] = $default_value ; 
  }
  return $q ; 
}

function _gather_form($form, $names)
{
  $q = array();
  foreach ($names as $name => $default_value)
  {
    if (!empty($form[$name]))
      $q[$name] = $form[$name] ;
    else 
      $q[$name] = $default_value ; 
  }
  return $q ; 
}



?>
