<?php

/**
 * $Id: carbon_chart.inc,v 1.9 2009/01/06 22:19:26 johnackers Exp $ 
 * 
 * chart functions for drupal carbon module. 
 * 
 * Created on 29-jan-2006
 */

/**
 * append links for available charts for this data
 */

function carbon_chart_link(&$links) 
{
  $query = $_GET['q'];    	
  $links["bar_chart"] = array("title"=>"bar chart", "href"=>$query, "query"=>"chart=bar", 
  "attributes" => array("title"=>"add a bar chart to this page"));   
  $links["pie_chart"] = array("title"=>"pie chart", "href"=>$query, "query"=>"chart=pie", 
  "attributes" => array("title"=>"add a pie chart to this page"));     
}

/**
 * if the user has requested a chart in the query 
 * embed the IMG tags into the HTML  
 */

function carbon_chart_embed($default, $firstdate)
{  
  $chart = null ;
  if (isset($_GET['chart']))
  { 
    $chart = $_GET['chart'];
  }
  else  // enhancement request - http://drupal.org/node/125959
  {     // allow the default chart type to be specified    
    $chart = $default;
  }
  
  if (!empty($chart))
  {
    $query = $_SERVER["QUERY_STRING"] ;  // does this work with all servers, prefer to use drupal fn?
    $link = "?".$query."&render=".$chart ;    
    if (isset($firstdate))
      $link .= "&firstdate=".$firstdate ;    
    return "<a href='".$link."'><img src='".$link."' alt='".$chart." chart not loaded' style='margin:10px'></a>" ; 
  }   	  
}


/**
 * Is this a request to render a particular chart
 */

function carbon_chart_requested()
{
  return isset($_GET["render"]);
}

/**
 * get the start date which should be embedded in request
 */

function carbon_chart_get_startdate()
{
  return ($_GET["firstdate"]);
}


/**
 * generate an image that contains the chart that had been  
 * embedded in the HTML
 * 
 * @param an array containing multiple $key => $values where key 
 * is the name of the item and value is size of the item 
 */


function carbon_chart_generate($map, $title)
{
  if (!function_exists("imagecreate"))
  {
  	return("function imagecreate not available, check that gd is installed");
  }  
	//  $map = array("abc" => 344, "def" => 432);

  $chart = $_GET['render'];  
  // make up the function name based on the render request
  $fn = "carbon_".$chart."_chart"; 


  if (function_exists($fn))
  {
    return $fn($map, $title);
  }
  else 
    return("This function does not exist: ". $fn);   
}


/**
 * create a bar chart as a png/image
 * @param an array of $title => $amount
 */


function carbon_bar_chart($dataset, $title)
{
  // allow the chart to create a large vertical (scrollable) image
  carbon_bar_chart_horiz($dataset, $title, 100, 50, 560, 10000);
}


/**
 * bars go left to right
 */

function carbon_bar_chart_horiz($dataset, $title, $min_width, $min_height, $max_width, $max_height)
{
  if (empty($dataset))
    return "empty array";   	
  
  $chart = new HorizontalBarChart(); 
  $chart->min_width = $min_width ;   
  $chart->max_width = $max_width ;  
  $chart->min_height = $min_height ;  
  $chart->max_height = $max_height ;      

  $chart->initialize($title, $dataset); 
  $image = imagecreate($chart->sizex, $chart->sizey);
  define_palette($image);

  $chart->posx = $chart->posy = 0 ; 
    
  $colorWhite = imagecolorallocate($image, 255, 255, 255);     
  imagefilledrectangle($image, 0, 0, $chart->sizex, $chart->sizey, $colorWhite);

  header("Content-type: image/png");
  $chart->render($image); 
  imagepng($image); 
  imagedestroy($image);    
}


class HorizontalBarChart
{ 
  var $min_width, $min_height, $max_width, $max_height ; 
  var $width ; 
  
  var $title, $titleheight = 20 ; 

  var $labelMaxWidth = 140 ;  // large enough to accomodate the key text
  var $labelHeight = 20 ;  // the sector logos 
  
  var $barWidth = 30 ;     // includes inter bar gap 
  var $barGap = 4 ;
  var $barMaxLength = 0 ; 
  var $dataset ;

  var $baseWidth = 3 ;       // cosmetic only
  var $baseLength ; 

  var $sizex, $sizey ;       // calculated on initialize
  var $posx, $posy  ;        // set by external functions to position object
   
  var $horizontalGap = 20 ;
  var $verticalGaps = 10 ;  
  // work out sizes 
  
     
  function initialize(&$title, &$dataset)
  {
    $this->title = $title ;
    $this->dataset = $dataset;  
    
    $width = $this->barWidth * count($dataset);
    $barMaxHeight = $this->max_height - $this->labelHeight ; 
    $barMinHeight = $this->min_height - $this->labelHeight ;  
   
    // adjust the bar width if the calculated width is very small
         
    if ($width < $barMinHeight) 
      $this->barWidth = $barMinHeight / count($this->dataset); 
      
    // of the height is enormous, actually would be 
    // better to stop adding images.
       
    if ($width > $barMaxHeight) 
      $this->barWidth = $barMaxHeight / count($this->dataset);     
    
    $this->baseLength = $this->barWidth * count($this->dataset); 

    $this->sizex = $this->max_width ;   // use available space for now
    $this->barMaxLength = $this->sizex - $this->baseWidth - $this->labelMaxWidth - $this->horizontalGap; 
    
    $this->sizey = $this->titleheight 
    + $this->verticalGaps + $this->baseLength 
    + $this->verticalGaps + $this->labelHeight; 
    
//    $this->title = "sizey ".$this->sizey." baselen ".$this->baseLength ;
  }

  function render(&$image)
  {
    global $customPalette, $black; 
  
    // write the title 
    imagestring($image, 3, $this->posx, $this->posy + $this->titleheight/8, $this->title, $black);
    imagestring($image, 2, $this->posx, $this->posy + $this->titleheight/2, $this->comment, $black);
   
    
    $this->posy += $this->titleheight + $this->verticalGaps; // move to top of bars
    $colNext = 0 ; 
    $shadow = false ;

    $bareBarWidth = $this->barWidth - $this->barGap ; 
          
    $max_value = 0 ; 

    $basex =  $this->labelMaxWidth + $this->horizontalGap ; 
    $barx = $basex + $this->baseWidth; 
     ;
    foreach($this->dataset as $key => $ds)
    {   
    	$value = $ds["data"];
      
      if (is_array($value))     
        $subtotal = array_sum($value);
      else
        $subtotal = $value ; 
      $max_value = max($max_value, $subtotal);  
    }  
    $scaling = $max_value == 0 ? 1 : ($this->barMaxLength)/$max_value;  
  
    // put down a base line      

    $colorBase = imagecolorallocate($image, 128, 128, 128);  
    imagefilledrectangle(
      $image, $basex, $this->posy - $this->barGap, 
      $basex + $this->baseWidth, $this->posy + $this->baseLength, 
      $colorBase);
  
    // Create bars
  
    $barNum = 0 ; 
    foreach ($this->dataset as $outerKey => $outerValue)
    { 
      $value = $outerValue["data"];
      $comment = $outerValue["comment"] ; 
      
      $sum = 0 ;
      $innerDataset = null ;           
      if (is_array($value))
      {
        $innerDataset = $value;
        $sum = array_sum($innerDataset);    
      }
      else
      {
        // create an inner dataset of size 1         
        $innerDataset = array();
        $innerDataset[$outerKey] = $sum = $value ;         
      }
      //
      // write a set of bars, one extending the other
      // 
      $x = $barx ; 
      $y = $this->posy + $barNum*$this->barWidth; 
      create_palette($innerDataset, $image);
                
      foreach ($customPalette as $key=>$col) 
      {           
        if (!isset($innerDataset[$key]))
          continue ; 
        $value = $innerDataset[$key];
        $barLength = $value * $scaling ; 
  
        // right_side, top, left_side, bottom
  
        imagefilledrectangle($image, $x, $y, $x += $barLength, $y+$bareBarWidth, $col);
        $x += 2 ;   // leave a gap between the bars
  
   //     if ($shadow)
   //       imagefilledrectangle($image, $baseWidth, $y = $i*$barWidth+$bareBarWidth, $baseWidth+$barLength-3,  $y += $shadow, $darkgray);
          
      }
      imagestring($image, 2, 0, $y , $outerKey." - ".sprintf("%.0f", $sum), $black);
      imagestring($image, 2, 0, $y + 12, $comment, $black);

      $barNum++ ; 
    }
    // write the color keys below the bars
    
    $labelx = $barx  ; 
    $squareSize = 10 ; 
    $y += $this->barWidth + $this->verticalGaps ; 
    foreach ($customPalette as $name => $col)
    {
      imagefilledrectangle($image, $labelx, $y, $labelx+$squareSize, $y+$squareSize, $col) ;   
      imagestring($image, 2, $labelx + 15, $y, $name, $black);
      $labelx += + $squareSize + strlen($name) * 6 + 15  ; 
    }     
  }
}




function carbon_bar_chart_vert($dataset, $title, $min_width, $min_height, $max_width, $max_height)
{
  $labelWidth = 200 ; //  large enough to accomodate the key text
  $barWidth = 40 ;    // start with arbitrary width  
  
  // work out sizes 
  $width = $barWidth * count($dataset);
  if ($width < $min_width) 
    $barWidth = $min_width / count($dataset); 
  if ($width > $max_width) 
    $barWidth = $max_width / count($dataset);     
  
  $imgWidth = $barWidth * count($dataset); 
  $imgHeight = $max_height ; 
    
  header("Content-type: image/png");

// Create image and define colors

  $image=imagecreate($imgWidth, $imgHeight);
  $colorWhite=imagecolorallocate($image, 255, 255, 255);
  $colorGrey=imagecolorallocate($image, 192, 192, 192);
  $colorBase=imagecolorallocate($image, 128, 128, 128);

  $colorFG=imagecolorallocate($image, 64, 64, 64);
  $colorBG=imagecolorallocate($image, 192, 192, 192);

  $baseHeight = 2 ;   
  $bareBarWidth = $barWidth ; 
  $barBottom = $imgHeight - $baseHeight - 1;
  $barTop = $labelWidth ; 
  
  $scaling = max($dataset) == 0 ? 1 : ($barBottom - $barTop)/max($dataset);

  // put down a base line  

  imagefilledrectangle($image, 0, $imgHeight -$baseHeight, $imgWidth, $imgHeight, $colorBase);

  // Create bar charts

  $i = 0 ;
  foreach ($dataset as $key => $value){
            
    $barHeight = $value * $scaling ;   
    // right_side, top, left_side, bottom
    imagefilledrectangle($image, $i*$bareBarWidth, $barBottom-$barHeight, ($i+1)*$bareBarWidth, $barBottom, $colorFG);
    imagefilledrectangle($image, ($i*$bareBarWidth)+1, $barBottom-$barHeight+1, (($i+1)*$bareBarWidth)-5, $barBottom, $colorBG);
    imagestringup($image, 2, $i*$bareBarWidth + $bareBarWidth/2 - 10, $barBottom-$barHeight-10, $key." - ".sprintf("%.0f", $value), 255);
    $i++ ;
  }
  // Output graph and clear image from memory
  
  imagepng($image); 
  imagedestroy($image);    
}



/**
 * 
 * create a pie chart as a png/image
 * @param an array of $title => $amount
 */


function carbon_pie_chart($outerset, $toptitle)
{
  global $white, $black, $lightgray ; 
  
  $charts = array();
  $imagex = 0 ; 
  $imagey = 0 ; 
  $x = 0 ; 
  $y = 0 ;
  $maxwidth = 480 ;

  $y = 40 ; // make room for title 
  $verticalGap = 10 ; // between charts
  $i = 0 ;
  foreach ($outerset as $title => $dataset)
  {
    $chart = new PieChart();  
    $legend = new Legend(); 

    // calculate the sizeof the image and legend
    $chart->initialize($title, $dataset); 
    $legend->initialize($dataset);
    
    // poke values into class members for desired position 
    // of this pie in the image. First try to put new image 
    // on the same line
    
    if ($x + max($chart->sizex, $legend->sizex) < $maxwidth)
    {
      $chart->posx = $legend->posx = $x ;
      $chart->posy = $y ; 
      $legend->posy = $y + $chart->sizey;            ; 
    } 
    else  // else move to the next line
    {
      $chart->posx = $legend->posx = $x = 0 ;     
      $chart->posy = $y = $imagey ;
      $legend->posy = $y + $chart->sizey ; 
    }
    $x += 10 + max($chart->sizex, $legend->sizex);
          
    $imagex = max($imagex, $x ); 
    $imagey = max($imagey, $legend->posy + $legend->sizey + $verticalGap);
    
    $charts[] = $chart ;    // save for rendering
    $charts[] = $legend ;   // save for rendering
  }
  
  $image = imagecreate($imagex+1, $imagey+1); 
  
  define_palette($image); 
  imagefilledrectangle($image, 0, 0, $imagex, $imagey, $white) ;

  imagestring($image, 5, 0, 15, $toptitle, $black);  
  
  $i = 0 ; 
  foreach ($charts as $chart)
  {    
  // debug - draw an outline around the box we are going to fill 
    imagerectangle($image, $chart->posx, $chart->posy, 
      $chart->posx+$chart->sizex, $chart->posy+$chart->sizey, $lightgray) ;
    $chart->render($image); 
  }
  
  
  // flush image
  header('Content-type: image/png');
  imagepng($image);
  imagedestroy($image);
} 

/**
 * create a basic color palette to use with pie charts 
 * (this may not be necessary, it seems everyone would need
 * to do this)
 */


function define_palette($image)
{
// allocate the CGA 16 color palette
  global $standardPalette, $sparePalette, $black, $white, $lightgray ;
  if (isset($standardPalette) && isset($sparePalette))
    return ;
    
  $black    = imagecolorallocate($image, 0x00, 0x00, 0x00);
  $blue     = imagecolorallocate($image, 0x00, 0x00, 0xAA);
  $green    = imagecolorallocate($image, 0x00, 0xAA, 0x00);
  $cyan     = imagecolorallocate($image, 0x00, 0xAA, 0xAA);
  $red      = imagecolorallocate($image, 0xAA, 0x00, 0x00);         
  $magenta  = imagecolorallocate($image, 0xAA, 0x00, 0xAA);
  $brown    = imagecolorallocate($image, 0xAA, 0x55, 0x00);
  $lightgray= imagecolorallocate($image, 0xF8, 0xF8, 0xF8);
  $darkgray = imagecolorallocate($image, 0x55, 0x55, 0x55);  
  $b_blue   = imagecolorallocate($image, 0x55, 0x55, 0xFF);
  $b_green  = imagecolorallocate($image, 0x55, 0xFF, 0x55);
  $b_cyan   = imagecolorallocate($image, 0x55, 0xFF, 0xFF);
  $b_red    = imagecolorallocate($image, 0xFF, 0x55, 0x55);  
  $b_magenta= imagecolorallocate($image, 0xFF, 0x55, 0xFF);
  $yellow   = imagecolorallocate($image, 0xFF, 0xFF, 0x55); 
  $grey     = imagecolorallocate($image, 0x80, 0x80, 0x80);
  $white    = imagecolorallocate($image, 0xFF, 0xFF, 0xFF);


  $sparePalette = array(
               $brown, $cyan, 
               $b_red, $b_green, $b_blue,  
               $b_magenta,  $yellow, $b_cyan, $black);
        
  $standardPalette = array(
        "heating" => $red, 
        "elec" => $blue, 
        "pt" => $green, 
        "car" => $magenta, 
        "air" => $b_blue,
        "mixed" => $grey);
}

/**
 * create a custom palette to match the keys provided in the 
 * dataset. Where keys match known keys, use a preferred colour.
 * Then use colours from the spare palette. 
 * 
 * Once all the colours are used, don't show anything else.
 */


function create_palette($dataset, $image)
{
	global $standardPalette, $sparePalette, $customPalette;
     
  define_palette($image);

  if (!isset($customPalette))
    $customPalette = array();
    
  $keys_not_assigned = array_merge($dataset);

  // have already allocated a colour for this key
  
  foreach($customPalette as $key => $col)
  {
    if (isset($keys_not_assigned[$key]))
    {
      unset($keys_not_assigned[$key]);
    }
  }
  
  foreach($standardPalette as $key => $col)
  {
    if (isset($keys_not_assigned[$key]))
    {
    	$customPalette[$key] = $standardPalette[$key];
      unset($keys_not_assigned[$key]);
    }
  } 

  $i = 0 ;  foreach($keys_not_assigned as $key => $object)
  {
    while ($i < count($sparePalette))
    {
    	$col = $sparePalette[$i++];
      if (in_array($col,$customPalette))
        continue;   
      $customPalette[$key] = $col ;  
      unset($keys_not_assigned[$key]);
    }
  }     
}

/** 
 * It is very painful using objects in PHP that has to run under version 4 PHP. 
 * To difference between 4 and 5 as described by others is that 4 passes objects
 * by value and 5 copies objects by reference. 
 * 
 * However it is very useful when you rendering different shapes of objects to 
 * use objects. There are two methods used to display objects
 *
 * This class draws a pie chart.
 *  
 * size() - allow the object to work out how much space it will take up on 
 * the canvas
 * 
 * render() - draw the object on the canvas.
 *  
 */


class PieChart 
{
  var $sizex, $sizey ;       // calculated on initialize
  var $posx, $posy  ;        // set by external functions to position object
  var $title, $titleheight ;
  var $comment ; 
  var $dataset ; 
  var $ak ;
  var $radiusx = 60 ;
  var $radiusy = 60 ;
    
  function initialize(&$title, &$dataset)
  {
    $this->title = $title; 
    $this->dataset = $dataset["data"]; 
    $this->comment = $dataset["comment"]; 
    // create image
    $this->ak = array_keys($this->dataset);
    $this->titleheight = 30 ; 
    $this->sizey =  $this->titleheight + $this->radiusy * 2 ; 	
    $this->sizex =  $this->radiusx * 2 ; 
  }	    
  
  function render(&$image)
  {
    global $customPalette, $black; 
    $total = 0 ; 
    
    // write the title 
    imagestring($image, 3, $this->posx, $this->posy + 0, $this->title, $black);
    imagestring($image, 2, $this->posx, $this->posy + $this->titleheight/2, $this->comment, $black);
    $this->posy += $this->titleheight  ; 
    
    // work out how many degrees each piece of the 
    // pie can be, making a total of 360 degrees. 
   
    foreach ($this->dataset as $key => $value)
    {
      $total += $value ;
    }   
    $scaling = ($total == 0) ? 0 : 360/$total;     
    
    $i = 0 ;
    $last = 0 ; 
  
//    imagestring($image, 3, 20, 20, $this->ak[0], $black);       
  
    create_palette($this->dataset, $image);
                
    foreach ($customPalette as $key=>$col)  
    {   
      if (!isset($this->dataset[$key]))
        continue ; 
      $value = $this->dataset[$key];
      $next = $value * $scaling + $last;
 
      //  draw the pie 
      //  cx, cy, w, h, s, e, col, 
      imagefilledarc($image, 
        $this->posx + $this->radiusx, $this->posy + $this->radiusy, 
        2 * $this->radiusx, 2*$this->radiusy, 
        $last, $next, 
        $col, IMG_ARC_PIE);
          
      $last = $next ;    $i++ ; 
    }        	
  }      
}

/** 
 * This class writes the legend for a pie chart or a bar chart
 */


class Legend 
{
  var $sizex = 0, $sizey = 0 ; 
  var $posx, $posy	;

  var $legend_height ; 
  var $dataset ; 
  
  function initialize(&$dataset)
  {
    global $customPalette;
    $this->dataset = $dataset["data"] ; 
    $this->legend_height = 18; 
    $this->sizex = 150 ;
      
    foreach ($this->dataset as $key=>$val)  
    {   
      if (empty($val)) 
        continue ;
      $this->sizey += $this->legend_height;
    }
  }

  function render(&$image)
  {
    global $customPalette, $black;
    $i = 0 ;  
    foreach ($customPalette as $key=>$col)  
    {   
    	if (empty($this->dataset[$key])) 
        continue ; 
      $y = $this->posy + ($i * $this->legend_height) ; 
  
      // draw the key 
      imagefilledrectangle($image, $this->posx, $y, $this->posx+10, $y+10, $col);
      // 
      // now write the entry into the keys. 
      //  
      imagestring($image, 2, $this->posx + 20, $y, $key." - ".sprintf("%.0f", $this->dataset[$key]), $black);       
      $i++ ; 
    }  	
  }  
}


?>