<?php
/** 
 * $Id: carbon_stamp.inc,v 1.21 2009/01/06 22:19:26 johnackers Exp $ 
 * 
 * Drupal carbon stamp module.
 * 
 * A carbon stamp is a record of an activity that resulted in CO2 emission. 
 *
 * Created on 18-Oct-2006
 *
 */


/**
 * Implementation of hook_access().
 * 
 * Delegate carbon_access to the carbon module because it 
 * is easier to check access for all carbon types in one place.  
 */

function carbon_stamp_access($op, $node)
{
  return carbon_access($op, $node);
}


/**
 * Render a page listing a one line summary of all carbon footprints 
 */


function carbon_stamp_page($uid) {
  global $user;

  $header = array(
    array("data" => t("title"),   "field" => "title"),
    array("data" => t("start"),   "field" => "startdate"),
    array("data" => t("end"),     "field" => "enddate"),
    array("data" => t("code"),    "field" => "code"),
    array("data" => t("amount"),  "field" => "reading"),
    array("data" => t("fp's"),    "field" => "numfoot"),
    array("data" => t("status"),  "field" => "status"),
    array("data" => t("operations"))
  );
  
  if (empty($uid))
    $header[] = array("data" => t("author"),"field" => "name");

  $sql = "SELECT  t.*, n.title, n.nid, n.status, n.uid, n.type, name, count(*) as numfoot, " .
         "BIT_OR(POWER(2,a.protection)) as protect2, j.sid " .
  		   "FROM {carbon_stamp} t " .
         "INNER JOIN {node} n                   ON t.nid = n.nid " .
         "INNER JOIN {users} u                  ON n.uid = u.uid " .
         "LEFT  JOIN {carbon_stamp_account} j   ON t.nid = j.sid " .         
         "LEFT  JOIN {carbon_account} a         ON j.fid = a.nid " .
         ((empty($uid) ? '' : " where n.uid = ".$uid. " ") . 
         "GROUP BY t.nid "); 
  
  $sql .= tablesort_sql($header); 
  
  // get the links (no range limit here)
  $queryResult = db_query($sql);

	$sampleRecord = null ;
  while ($node = db_fetch_object($queryResult)) 
  {
    $node->protection = _log2($node->protect2); 
    $ro_access = carbon_access("view", $node); 
    if ($ro_access)
    {
      $row =  array(carbon_link_from_title($node, $node),
				$node->startdate == 0 ? '' : _format_date($node->startdate, "small"),
				_format_date($node->enddate,"small"),
				$node->code, 
				$node->reading.' '._get_scope_key($node->scope),
				$node->numfoot <= 1 ? (isset($node->sid) ? 1 : 0) : $node->numfoot, 		
				$node->status >= 1 ? 'pub' : '',
        $node->protection . " " . carbon_format_edit_ops($node, null));
    
      if (empty($uid))
        $row[] = $node->name ;
      $rows[] = $row ;  
    }
    $sampleRecord = $node ;  
  }
  $output = theme("table", $header, $rows);
  $output .= theme("pager", NULL, 10, 0);

  $title = t("Carbon Stamps");
   
  if (!empty($uid)) 
    $title .= t(" owned by !name", array("!name" => $sampleRecord->name));
     
  drupal_set_title(check_plain($title));      
  
  if (isset($sampleRecord))
  {
    $links = carbon_link("node", $sampleRecord); 
    $output .= theme_links($links);    // format links
  }  
  return $output;

}

function _log2($protect2)
{
	$k = 0 ;
	for ($i = 1 ; $i < $protect2 ; $i *= 2)
    $k++ ; 
  return $k  ; 
}



function carbon_stamp_header()
{
  global $carbon_stamp_header ;
  if (!isset($carbon_stamp_header))
  { 	 
    $carbon_stamp_header = array(
    array("data" => t("date"), "field" => "enddate", "sort" => "asc"),
    array("data" => t("title"), "field" => "sourcetitle"),
    array("data" => t("model"), "field" => "sourcemodel"),
    array("data" => t("emission"),"field" => "title"),
	  array("data" => t("amount"), "field" => "reading"),
	  array("data" => t("units"),  "field" => "units"),
	  array("data" => t("adjust"), "field" => "adjustment"));
  }
	//    array("data" => t("type or size")),
  return $carbon_stamp_header ; 
}

/**
 * 
 * Joins the recorded stamps in a carbon account with all the available carbon 
 * sources. 
 * 
 * Note that some source fields have been renamed e.g. sourceid, sourceclass, sourcetitle 
 *
 *@param $nid  select stamps attached to this footprint id, can be null
 *@param $uid  select stamps with this uid, can be null  
 *@param $org  select stamps with this org, can be null  
 *@return an array of carbon stamps (and an array of carbon sources) 
 */


function carbon_stamp_array_load($nid, $uid, $org)
{
  /** 
   * this is a very tricky SQL statment. It is currently arranged to 
   * show all carbon stamps that do not have a matching carbon source
   * so that user can be warned. 
   */

	$query ="SELECT  nstamp.title, nstamp.status, nstamp.uid, nstamp.type, stamp.*, s.uid AS shared_uid, " .
    "nsource.title as sourcetitle, source.nid as sourceid, source.code as sourcecode, ".
    "source.classname as sourceclass, nsource.status as sourcestatus, ".
    "source.sector, source.model as sourcemodel, units, tokwh0, tokwh1, toco2, params, " .
    "f.model as preferred_model, f.organization, f.enablekwh0, f.enablekwh1, f.enableco2 " .
    "FROM {carbon_stamp} stamp " .
    "LEFT JOIN {carbon_source} source ON stamp.code=source.code " .
    "LEFT JOIN {carbon_stamp_account} j ON stamp.nid = j.sid " .	
    "INNER JOIN {carbon_account} f      ON     f.nid = j.fid " .
    "LEFT JOIN {carbon_account_share} s ON     f.nid = s.nid " . 
    "INNER JOIN {node} nstamp ON  stamp.nid = nstamp.nid ". 
    "LEFT JOIN {node} nsource ON  source.nid = nsource.nid ".            		
    "WHERE   nstamp.status ".      // stamp must be published  
 //   "AND nsource.status ".         // source must be published   
    "AND (j.fid = %d  " .           (empty($nid) ?  " OR 1) " : ") ").
    "AND (nstamp.uid = %d " .       (empty($uid) ?  " OR 1) " : ") ").           
    "AND (f.organization = '%s' " . (empty($org) ?  " OR 1) " : ") ").  
    "" ;   

//	$query = db_rewrite_sql($query); 
//  drupal_set_message(" my sql ". $query, "info"); 

  $query .= tablesort_sql(carbon_stamp_header()); 
  $queryResult =  db_query($query, $nid, $uid, $org);

  $stamps = array(); 	
  $sources = array();
  $num = 0 ; 
	while ($record = db_fetch_object($queryResult))
	{
    $stamps[] = $record ; 
/*
		if ($record->nid != null)
    {
//      drupal_set_message("pref model ". $record->preferred_model. "  src model ". $record->sourcemodel, "info");   
      // we don't join the footprint preferred model with the source model because the
      // source may not be available for the selected model. Instead walk through the rows 
      // and pick the first row that matches by code only and replace that with a model 
      // match if it happens to occur.   
      if ($record->sourcemodel == $record->preferred_model || !isset($stamps[$record->nid]))
        $stamps[$record->nid] = $record ;            
    }		
    $sources[$record->nid] = $record ;
*/

	}   
  $filtered_stamps = _select_stamps_from_preferred_sources($stamps);

	return array("source" => array("no longer", "gathered"), "stamp" => $filtered_stamps);
}




function _stamp_form(&$form, $source_keys, $source_descriptions, $stamp, $i, $group, $group_title)
{
  $stamp_group_form = "g".$group ; 
	$stamp_form = "f".$i ; 

	  if (empty($form[$stamp_group_form]))
	  {
		  $form[$stamp_group_form] = array(
				"#type" => "fieldset", 
				"#title" => $group_title, 
	    		"#weight" => 1,				
				"#collapsible" => TRUE,
				"#collapsed" => FALSE);
	  }

	  $form[$stamp_group_form][$stamp_form] = array(
			"#type" => "fieldset", 
			"#collapsible" => TRUE,
			"#collapsed" => false);

	  $form[$stamp_group_form][$stamp_form]["ff".$i."title"] = array(
		  "#type" => "textfield", 
    	"#title" => t("title"),  
	    "#required" => FALSE, 
 	    "#default_value" => empty($stamp->title) ? "My carbon emissions" : $stamp->title ,
      "#size" => 30,
      "#maxlength" => 40, 
	    "#weight" => -2,
	    "#description" => 
          _div_wrap("scope-0",t("Name of activity e.g. train to Leeds")).
          _div_wrap("scope-1",t("Meter id e.g. gas. " .
          "This must be the same for all readings from the same meter.")).
          _div_wrap("scope-2",t("Name of activity"))
          );	
	  
	  $form[$stamp_group_form][$stamp_form]["ff".$i."scope_index"] = array(	
      "#attributes" => array("onchange" => "scope_change(this.value,'slow');"),    
		  "#type" => "radios", 
	    "#title" => t("type of carbon debit"), 
	    "#required" => FALSE, 
		  "#options" => _get_scope_description(),
      "#size" => 14,
      "#maxlength" => 20, 
	    "#weight" => -1,
	    "#default_value" => empty($stamp->scope) ? 0 : $stamp->scope,
	    "#description" => 
          _div_wrap("scope-0",t("A one off emission can be used for single journeys ")).
          _div_wrap("scope-1",t("Note that meter readings are subtracted from one another " .
          "to calculate consumption. You need to have at least two meter readings before this " .
          "source of emissions will be included in your carbon footprint. You cannot specify " .
          "start or end dates.")).
          _div_wrap("scope-2",t("Record a regular emission such as a daily commute "))          
          );	    
	    
	  $form[$stamp_group_form][$stamp_form]["ff".$i."code_index"] = array(
		  "#type" => "select", 
    	"#title" => t("select type of carbon debit"),  
	    "#required" => FALSE, 
		  "#options" => $source_descriptions,   
	    "#default_value" => empty($stamp->code) ? 0 : array_search($stamp->code,$source_keys),
      "#size" => 1,
      "#maxlength" => 20, 
	    "#weight" => 0,
	    "#description" => "<div class='scope-0'>".t("Enter the one off value")."</div>". 
                        "<div class='scope-1'>".t("Enter the meter reading")."</div>");
  	  
	  $form[$stamp_group_form][$stamp_form]["ff".$i."reading"] = array(
		  "#type" => "textfield", 
    	"#title" => t("amount"),  
	    "#required" => FALSE, 
	    "#default_value" => empty($stamp->reading) ? "" : $stamp->reading, 
      "#size" => 14,
      "#maxlength" => 20, 
	    "#weight" => 0,
	    "#description" => t("Number of units, miles, gallons, litres, KwH etc. ") .
	       t("Calculate distance between ").
         l(t("airports around the world"), "http://www.timeanddate.com/worldclock/distance.html",
         array("TARGET" => "distance_calculation_global"))." or between ".
         l(t("local towns"), "http://co2balance.uk.com/co2calculators/rail-travel/",
         array("TARGET" => "distance_calculation_local"))."."
         );

	  $form[$stamp_group_form][$stamp_form]["ff".$i."startdate_s"] = array(
		  "#type" => "textfield", 
    	"#title" => t("starting date"),  
	    "#required" => FALSE, 
	    "#default_value" => empty($stamp->startdate)  ? "" : _format_date($stamp->startdate, "custom", "Y-m-d H:i:s"),
      "#size" => 14,
      "#maxlength" => 20,       
	    "#weight" => 1,
      "#prefix" => "<div class='startdate'>",
      "#suffix" => "</div>",
	    "#description" => 
	    		t("Format: %time.", 
	    		array("%time" => _format_date(time(), "custom", "Y-m-d"))).
          _div_wrap("scope-0",t("The first date of the activity. Leave blank".
          " except when you are specifying an activity that spreads over days or weeks".
          " such as the total distance travelled by bus over the last month or two.")).
          _div_wrap("scope-1",t("Not used")).
          _div_wrap("scope-2",t("The day after the last date on which this repeating event stopped")));
	    		
	  $form[$stamp_group_form][$stamp_form]["ff".$i."enddate_s"] = array(
		  "#type" => "textfield", 
    	"#title" => t("date"),  
	    "#required" => FALSE, 
	    "#default_value" => $stamp->enddate == 0 ? "" : _format_date($stamp->enddate, "custom", "Y-m-d H:i:s"),
      "#size" => 14,
      "#maxlength" => 20, 
	    "#weight" => 2,
	    "#description" => 
	    		t("Format: %time. Leave blank to use today's date. ", 
	    		array("%time" => _format_date(time(), "custom", "Y-m-d"))).
          _div_wrap("scope-0",t("The last date of the activity")).
          _div_wrap("scope-1",t("The date of the meter reading")).
          _div_wrap("scope-2",t("The day after the last date on which this repeating event stopped")));

	  
    $help_shared_heating = "<h4>Sharing heating, electricity or a car trip</h4>" .
          "For example if you are sharing with 4 people you might enter <b>1/4</b>.";
    
    $help_repeating_trips = 
          "<h4>Two way trip</h4>Enter <b>2</b> if this is a two way trip. ".
          "<h4>Frequent Trip</h4>If you make a return trip 1 day a month, you might " .
          "enter <b>2/30</b>, or if its once a year you might enter <b>2/365</b>. " .
          "If this is a daily commute to work and you take 5 weeks holiday, you " .
          "might want to enter <b>2*220/365</b>. " .
          "Don't forget to enter the (starting) date of the first trip."; 
       
	  $form[$stamp_group_form][$stamp_form]["ff".$i."adjustment"] = array(
		  "#type" => "textfield", 
    	"#title" => t("adjustment"),  
	    "#required" => FALSE, 
	    "#default_value" =>  $stamp != null && $stamp->adjustment != 0 ? $stamp->adjustment : 1, 
	    "#weight" => 3,
      "#size" => 10,
      "#maxlength" => 20, 
	    "#description" => t("This is the complicated bit. Normally 1 except when:").
          _div_wrap("scope-0",t($help_shared_heating)).
          _div_wrap("scope-1",t($help_shared_heating)).
          _div_wrap("scope-2",t($help_repeating_trips)));		     
	    
	  $form[$stamp_group_form][$stamp_form]["ff".$i."nid"] = array(
		  "#type" => "hidden", 
	    "#value" => $stamp->nid);	
	    
	  $form[$stamp_group_form][$stamp_form]["ff".$i."code"] = array(
		  "#type" => "hidden",
	    "#value" => null);
	
	return $form ; 
}

/**
 * $stamp is the actual stamp that we are trying to edit (null if creating)
 * $clone is the stamp that we want to clone and add as a new entry 
 * but with, say, a different meter reading 
 */


function _stamp_repeating_form(&$form, $stamp, $clone, $i, $group)
{
	$group_title = $stamp->title." +" ; // . " | " . $clone->code ;	

	$stamp_group_form = "g".$group ; 
  $stamp_form = "f".$i ; 
  
	  if (empty ($form[$stamp_group_form]))
	  {
		  $form[$stamp_group_form] = array(
				"#type" => "fieldset", 
				"#title" => $group_title,
        "#weight" => 0,	    					 
				"#collapsible" => TRUE,
				"#collapsed" => true);
	  }

	  if (empty($form[$stamp_group_form][$stamp_form]))
	  {
		  $form[$stamp_group_form][$stamp_form] = array(
				"#type" => "fieldset", 
//          "#title" => $title, 
//          '#prefix' => '<div class="container-inline">',
//          '#suffix' => '</div>',	
				"#collapsible" => false,
				"#collapsed" => false);
	  }

	  $form[$stamp_group_form][$stamp_form]["ff".$i."reading"] = array(
		  "#type" => "textfield",  
      "#title" => (isset($clone->scope) && $clone->scope == 1) ? "meter reading" : "amount",
	    "#required" => false, 
	    "#default_value" =>  empty($stamp) ?  "" : $stamp->reading, 
      "#size" => 20,
      "#maxlength" => 20,    
	    "#weight" => -1,
      "#description" => empty($clone) ? "amount" : $clone->sourcetitle. ", ".$clone->units,  );


	  $form[$stamp_group_form][$stamp_form]["ff".$i."enddate_s"] = array(
		  "#type" => "textfield", 
    	"#title" => t("date (or end date) of emission"),  
    	"#length" => 10,      	
	    "#required" => false, 
	    "#default_value" => empty($stamp->enddate) ? "" : _format_date($stamp->enddate, "custom", "Y-m-d"),
	    "#weight" => 0,
      "#size" => 10,
      "#maxlength" => 20,   
	    "#description" => $stamp != null ? "" : t("Format: %time. Leave blank to use today's date.", array("%time" => _format_date(time(), "custom", "Y-m-d H:i:s O"))));

	// all the other fields will be copied from the stamp with nid = "cloneid"		
		
	  $form[$stamp_group_form][$stamp_form]["ff".$i."cloneid"] = array(
		  "#type" => "hidden", 
	    "#value" => $clone->nid);

	  $form[$stamp_group_form][$stamp_form]["ff".$i."nid"] = array(
		  "#type" => "hidden", 
	    "#value" => $stamp->nid);
	    	
	   return $form ; 
}

/**
 * when a user creates or edits a carbon_stamp, s/he may want to 
 * add or remove it from a particular carbon footprint. This 
 * form is only attached when individual carbon stamps are 
 * edited.
 */

function _add_footprint_select(&$form, $node, $i, $group)
{
  global $user ; 
  $stamp_group_form = "g".$group ; 
  $stamp_form = "f".$i."footprint" ; 

  // find all the carbon footprints owned by the same user as this stamp
  // or , if this is a new stamp, the logged in user. This is really to
  // make testing easier.

  $uid = isset($node->uid) ? $node->uid : $user->uid ;  
  $queryResult = db_query("SELECT n.nid,n.title from {node} n ".
        " INNER JOIN {carbon_account} f ON n.nid=f.nid ".
        " WHERE type='carbon_account' and uid = ".$uid);  


  // get all the normal attachments     
  while ($record  = db_fetch_object($queryResult)) 
  {  
    $options[$record->nid] = $record->title ;
  }  
    
  $ticked_boxes = is_array($node->fid_map) ? $node->fid_map : array(); 
  
  // add all the existing attachments which include footprints 
  // belonging to other UIDs. 
  
  foreach($ticked_boxes as $nid => $title)
    $options[$nid] = $title ; 
  
  $form[$stamp_group_form][$stamp_form] = array(
    "#type" => "fieldset", 
    "#weight" => 2,       
    "#title" => t("select footprint(s)"),                       
    "#collapsible" => TRUE,
    "#collapsed" => FALSE);  
     
      
  // this clumsy code sets the only check box if this is a new 
  // carbon stamp and there is only one footprint to attach to

  if (empty($node->nid) && empty($exist) && count($options) == 1)
  {
    $ticked_boxes = $options; 
  }
   
  // if there is only one option and it is ticked, collapse the frame
   
  if (count($options) == 1 && count($ticked_boxes) == 1)
  {
    $form[$stamp_group_form][$stamp_form]["#collapsed"] = true; 
  }

  $form[$stamp_group_form][$stamp_form]["fid_updated"] = array(
  "#type" => "checkboxes",
  "#default_value" => array_keys($ticked_boxes),
  "#options" => $options, 
  "#description" => t("Click on the footprints to which you would like to attach this carbon stamp.")
  );    
}


function carbon_stamp_form(&$node, &$param) {
	global $user, $fid_map;

	$source = carbon_source_fetch_array();
	$map = _get_source_map($source);
	$source_keys = array_keys($map); 
	$source_descriptions = array_values($map); 

  $form = array(); 
  
  if (count($source_descriptions) == 0) 
  {
    $form["no-sources"] = array(
      "#type" => "item", 
      "#value" => carbon_source_missing());  
    return $form ;  
  }

  /** 
   * allow one carbon stamp to be cloned and added as a new 
   * carbon stamp to the carbon footprint. 
   */

  if (isset($_GET['clone']))
  { 
    $id = $_GET['clone'];
    $clone = node_load($id);  $fid_map = array();  // wipe out footprint links
    
    // meter readings have to have the same title 
    // otherwise they are not treated as a group
    
    if ($clone->scope != 1)
      $clone->title = "copy of ".$clone->title ;   
    unset($clone->nid);   // make sure we don't overwrite the original 
    $node = $clone ;      
  }
  $form = _stamp_form($form, $source_keys, $source_descriptions, $node, 0, 0, $node->title);
  _add_footprint_select($form , $node, 0 , 0 ); 
  
  $path = drupal_get_path("module", "carbon");  
  drupal_add_js( $path . "/carbon.js", "module");
  drupal_add_css($path . "/carbon.css");
  
  return $form ;   
}


/**
 * scope is held as a integer in the database which 
 * indexes this map directly, so take care about changing the 
 * order of the entries in the table. 
 */

function _scope_map()
{
  return array(
    "" =>    t("a one off"),
    "m" =>   t("meter or mileometer reading"),
    "r" =>   t("repeats daily")
    );
}

/**
 * @return a short code/description of this numeric scope value
 */

function _get_scope_key($i)
{
  global $scope_keys ; 
  if (empty($scope_keys))
    $scope_keys = array_keys(_scope_map()); 
  return $scope_keys[$i] ;
}

/**
 * @return a long description of this numeric scope value
 */

function _get_scope_description($i = null)
{
  global $scope_values ; 
  if (empty($scope_values))
    $scope_values = array_values(_scope_map());   
  return $i == null ?   $scope_values : $scope_values[$i] ;
}

/**
 * @return a short description of this numeric scope value
 */


function _get_scope_short_description($i = null)
{
  if ($i == 0) return null ; 
  return _get_scope_description($i); 
}


/**
 * Validate our forms
 * @param string form id
 * @param array form values
 */
function carbon_stamp_validate($form_id, $form_values) 
{
	for ($i = 0 ; $i < 500 ; $i++)
  {
	  _validate_adjustment($form_id, "ff". $i .".adjustment");
    _validate_date($form_id, "ff". $i .".startdate_s"); 
    _validate_date($form_id, "ff". $i .".enddate_s"); 
//    _validate_date($form_id, "ff". $i .".firstdate"); 
  }
}


/**
 * the field is evaluated PHP so it is important not to 
 * allow any code to be maliciously inserted.
 */

function _validate_adjustment($form_id, $field_id)
{
  $matches = array();
  if (!empty($form_id->$field_id))  
    if (!preg_match('/^[0-9\.\*\/\+\-\(\)]*$/', $form_id->$field_id, $matches))
      form_set_error($field_id, t("Only . 0-9 ( ) * / + or - accepted in 'adjustment' field"));  	
}


function _validate_date($form_id, $field_id)
{
  $matches = array();
  if (!empty($form_id->$field_id))  
    if (!preg_match("/^\d\d\d\d\-\d\d\-\d\d/", $form_id->$field_id, $matches))
      form_set_error($field_id, t("Please check that date is in YYYY-mm-dd format.")); 
}



/**
 * This is a hook used by node modules. It is called after validation has 
 * succeeded and before insert/update. It is used to for actions which must 
 * happen only if the node is to be saved.
 */
function carbon_stamp_submit(&$node)
{			
	$source_keys = array_keys(_get_source_map(carbon_source_fetch_array()));
	_map_stamp_fields($node, $node, 0, $source_keys);
//  drupal_set_message()

  if (!isset($node->fid_updated))
  {
    drupal_set_message("Unable to attach to a footprint", "info");	
    return ; 
  }
  global $fid_map ; 
  if (!isset($fid_map))
    $fid_map = array();
     
  // after the editing, on checked boxed the values in 
  // fid_updated are set to the keys, on unchecked boxes
  // the values are set to zero.  

  $fid_updated = array_flip($node->fid_updated);
  if (isset($fid_updated[0])) 
    unset($fid_updated[0]);   // loose the 0 key created by any unchecked boxes
  $node->fid_insert = array_diff(array_keys($fid_updated), array_keys($fid_map));
  $node->fid_delete = array_diff(array_keys($fid_map), array_keys($fid_updated));  
}

/**
 * This function called locally and from carbon_account 
 * after footprint editing. 
 */

function _map_stamp_fields(&$dest, $src, $i, $source_keys)
{
	$fields = array("nid","cloneid","code_index","scope_index",
	"title","reading","adjustment","startdate_s","enddate_s","exclude");		
	
	// the source fields are mapped in a wack way so 
	// them onto the the destination object

	foreach ($fields as $f)
	{
		_map_form_field($dest, $src, $i, $f);
	}	

	$upd = 0 ;
	if (isset($dest->code_index))
	{
		$dest->code = $source_keys[$dest->code_index];  $upd += 1;
	}
	if (isset($dest->scope_index))
	{
		$dest->scope = $dest->scope_index;  $upd += 2;
	}
	if (!empty($dest->startdate_s))
	{
		$dest->startdate = strtotime($dest->startdate_s);  $upd += 4;
	}
	else 
	{
    $dest->startdate = "" ;    // remove existing date
	}  
	if (!empty($dest->enddate_s))
	{
		$dest->enddate = strtotime($dest->enddate_s);	$upd += 8;
	}



	if (empty($dest->enddate))
	{
		$dest->enddate = time();   $upd += 16;
	}
//	drupal_set_message('(debug) updated fields map :'.$upd, 'info'); 
}


function _map_form_field(&$dest, $src, $i, $field)
{
	$prefix = 'ff' ;	
	$fullname = $prefix.$i.$field ;

	_set_field($dest, $field, $src, $fullname); 

}


function _set_field(&$dest, $name, $src, $fullname)
{
	if (isset($src->$fullname))
		$dest->$name = $src->$fullname ; 			
	else 
	// and if the field is not present, set it (to null) but 
	// if the field has a non null value don't overide it with a null
		
		if (!isset($dest->$name))
			$dest->$name = $src->$fullname ; 					
}

/**
 * Implementation of hook_load.
 * Slightly cheekily get the count of the number of footprints in the query 
 * 
 */


function carbon_stamp_load(&$node)  {
		
	$query = db_query(
	"SELECT t.* , j.fid as attached_fid, n.title as attached_title " .
	"FROM {carbon_stamp} t " .
  "LEFT  JOIN {carbon_stamp_account} j ON t.nid = j.sid " .
  "LEFT  JOIN {node}                   n ON j.fid = n.nid " . // to get footprint title
	"WHERE t.nid = %d " .
	" ", $node->nid);
  $first_record = null ; 
  global $fid_map ; $fid_map  = array() ;  // having to hold this array because it is being wiped. 
  while ($record = db_fetch_object($query))
  {
  	if (empty($first_record))
    {
    	$first_record = $record ; 
      $first_record->fid_map = array();
    }
    if (!empty($record->attached_fid))
      $first_record->fid_map[$record->attached_fid] = $record->attached_title ; 
    $fid_map = $first_record->fid_map ; 
  }
  if (empty($first_record))
    drupal_set_message("Failed to load carbon_stamp where nid=", $node->nid);

	return $first_record;
}


/**
 * Implementation of hook_insert, which saves carbon-specific information
 * into the carbon table
 * @param node object
 */
 
function carbon_stamp_insert(&$node) 
{

  db_query("INSERT INTO {carbon_stamp} ".
	"(code, scope, reading, adjustment, protection, " .
	" exclude, startdate, enddate, nid) ".
	"VALUES ('%s', %d, '%s', '%s', %d, %d, %d, %d, %d)", 
	$node->code, $node->scope, $node->reading, $node->adjustment, 
	$node->protection, $node->exclude, $node->startdate, $node->enddate,
	$node->nid);   
  _update_footprint_join($node);   
}



/**
 * Implementation of hook_update, which saves updated todo-specific 
 * information into the todo table
 * @param node object
 */
function carbon_stamp_update(&$node) 
{

  db_query("UPDATE {carbon_stamp} set code='%s',scope=%d,reading='%s',adjustment='%s',protection=%d," .
    		" exclude=%d, startdate=%d, enddate=%d " .
    		" WHERE nid=%d", 
	$node->code, $node->scope, $node->reading, $node->adjustment,
	$node->protection, $node->exclude, $node->startdate, $node->enddate,
 	$node->nid);
  _update_footprint_join($node);   
}

/**
 * the use may choose to insert and delete the stamp from different 
 * footprints. carbon_stamp_submit() works out which tables need 
 * updating. This function inserts/removes the row.
 */


function _update_footprint_join($node)   
{	
  if (isset($node->fid_insert))   
    foreach ($node->fid_insert as $fid) 
  {
    db_query("INSERT INTO {carbon_stamp_account} (sid,fid) VALUES (%d, %d)", $node->nid, $fid);
  }

  if (isset($node->fid_delete))
    foreach ($node->fid_delete as $fid)    
  {
    db_query("DELETE FROM {carbon_stamp_account} WHERE sid=%d AND fid=%d", $node->nid, $fid);
  }
}


/**
 * Implementation of hook_delete()
 */
function carbon_stamp_delete($node) {
	db_query("DELETE FROM {carbon_stamp_account} WHERE sid = %d", $node->nid);   
	db_query("DELETE FROM {carbon_stamp}           WHERE nid = %d", $node->nid);
}


/**
 * Implementation of hook_view, add our node specific information
 * @param node object to display
 * @param boolean is this a teaser or full node?
 * @param boolean is this displaying on its own page
 * 
 */
function carbon_stamp_view(&$node, $teaser = FALSE, $page = FALSE) 
{
//	if (!user_access('view carbon footprints')) {
//		return true;
//	}

  $node = node_prepare($node, $teaser);
  $carbon_account_info = theme('carbon_stamp_table_view', $node); 
  $node->content["stamp"] = array("#value" => $carbon_account_info);   
  $node->body .= $carbon_account_info;
  $node->teaser .= $carbon_account_info;
  return $node ;
}



function theme_carbon_stamp_table_view($node)
{
  $header = array(
    array("data" => t("a")),
    array("data" => t("b"))
  );
  global $fid_map ; 
  $attached_footprint = '' ;
  foreach ($fid_map as $fid => $title)
  {
  	$attached_footprint .= l($title, "node/".$fid)." ";
  }

	// content variable that will be returned for display

	$rows[] = array("title", $node->title);
  $rows[] = array("model", $node->model);

	$rows[] = array("code", $node->code. "  ");
	$rows[] = array("amount",$node->reading );
	$rows[] = array("adjustment",$node->adjustment);
	$rows[] = array("scope", _get_scope_description($node->scope));
	$rows[] = array("start date", empty($node->startdate) ? "" : _format_date($node->startdate ));

	$rows[] = array("end date", _format_date($node->enddate ));
//$rows[] = array("exclude", $node->exclude);
	$rows[] = array("created", _format_date($node->created ));
	$rows[] = array("changed", _format_date($node->changed ));
	$rows[] = array("published", $node->status);
	$rows[] = array("footprints attached", $attached_footprint);
	
	$output = theme("table", null, $rows);

  return "<h1></h1>".$output;  // the H1s make bluebreeze theme work
}

/**
 * Implementation of hook_view, add our node specific information
 * @param node object to display
 * @param boolean is this a teaser or full node?
 * @param boolean is this displaying on its own page
 * 
 */
function carbon_stamp_edit(&$node, $teaser = FALSE, $page = FALSE) 
{
//  if (!user_access('view carbon footprints')) {
//      return true;
//  }

  $node = node_prepare($node, $teaser);
  $carbon_account_info = theme('carbon_stamp_table_view', $node);  
  $node->body .= $carbon_account_info;
  $node->teaser .= $carbon_account_info;
}


/**
 * Theme carbon table edit form  (this is not linked to menus yet)
 */
function theme_carbon_stamp_table_edit($form) {
  // Overview table:
  $header = array(NULL, t('Title'), t('Scope'), t('Code'), t('Reading'));
  $output = '' ; 
  $output .= form_render($form['account']);

    
  for ($g = 0 ;  ; $g++)
  {
      $rows = array(); 
      $stamp_group_form = "g".$g ; 
      if (!isset($form[$stamp_group_form])) 
        break ; 
        
      foreach (element_children($form[$stamp_group_form]) as $stamp_form) {
        $row = array();
        $subform = $form[$stamp_group_form][$stamp_form];
        $row[] = _form_render($subform[$stamp_form."title"]);
        $row[] = _form_render($subform[$stamp_form."code_index"]);
        $row[] = _form_render($subform[$stamp_form."reading"]);
        $row[] = _form_render($subform[$stamp_form."startdate_s"]);
        $row[] = _form_render($subform[$stamp_form."enddate_s"]);
        $rows[] = $row ;   
      }                      
      $output .= theme('table', $header, $rows);

  }
  $output .= form_render($form);
  if ($form['pager']['#value']) {
    $output .= form_render($form['pager']);
  }   
  return $output;
}


/**
 * remove all the text fields surround the box so it takes
 * up less space around 
 */

function _form_render($form)
{
  unset($form["#title"]);
  unset($form["#description"]);
  return form_render($form);	
}

function _div_wrap($wrap_class, $text)
{
	return "<div class='".$wrap_class."'>".$text."</div>" ; 
}



function _select_stamps_from_preferred_sources(&$stamps)
{
  $sourceWeight = new SourceWeight();
  $sourceWeight->initialize();
  $wstamps = array();
    
  foreach($stamps as $stamp)
  {
    $w = $sourceWeight->getWeight($stamp->sourcemodel); 
    if ($w < 1)  // if weight 1 or more means not found
      $wstamps[$stamp->nid][$w] = $stamp; 
    else
      $wstamps[$stamp->nid][] = $stamp ; 
  }

  $selected_stamps = array();
  foreach($wstamps as $nid => $stamp_ar)
  {
    ksort($stamp_ar);  // move the lightest weighted item to be first in the list
    foreach($stamp_ar as $stamp)
    {
      $selected_stamps[$stamp->nid] = $stamp ; // take the first stamp only
      break ;   	
    } 
  }     
  return $selected_stamps ; 
}

/**
 * this handles the weighting of different carbon_sources 
 * The admin determines the carbon sources weighting via 
 * settings. The first listed source is the heaviest. 
 */


class SourceWeight  
{
	var $weighted_filters = array();
  
  function initialize()
  {	
    $max = 20 ;
    $filter_s = variable_get("carbon_model_filter", "", $max);
    if ($filter_s == "")
      return ;
  
    $filters = explode(",", $filter_s, 20);  

    /** 
     * create an array consiting of model_name => weight
     */
    for ($i = 0 ; $i < count($filters) ; $i++)
    {
      $weight = -count($filters) + $i ;
      $this->weighted_filters[trim($filters[$i])] = $weight ;    
    }  
  }
  
  /** 
   * look up a sourcemodel and return a weighting for it
   */
  
  function getWeight($sourcemodel)
  {
//  	print_r($this->weighted_filters); 
    $val = $this->weighted_filters[$sourcemodel] ; 
   
    if (isset($this->weighted_filters[$sourcemodel]))
      return $this->weighted_filters[$sourcemodel] ; 
    return 1 ; 
  } 
}




?>