<?php
/** 
 * $Id$ 
 * 
 * Drupal carbon module.
 * 
 * Carbon transfers (purchases and sales of carbon) are recorded in  
 * carbon_transfers nodes. The functions in this file manage the 
 * database operations and reporting functions.  
 * 
 * Intended for (but not exclusively) CRAG use. 
 *
 * Created on Feb 2008.
 *
 */


/**
 * Implementation of hook_access().
 * 
 * Delegate carbon_access to the carbon module because it 
 * is easier to check access for all carbon types in one place.  
 */

function carbon_transfer_access($op, $node)
{
  return carbon_access($op, $node);
}


/**
 * Render a page listing a one line summary of all carbon transfers
 */


function carbon_transfer_page($account_id) {
  global $user;

  $headerAllAccounts = array("valuedate", "description", "seller", "buyer", "carbon", "cost", "author", "ops");

  $headerOneAccount = array("valuedate", "description", "counterparty", "credit", "debit", "balance", "author", "ops");
  
  $header = empty($account_id) ? $headerAllAccounts : $headerOneAccount ; 
  
  $account_map = new AccountMap() ; $account_map->init(); 

  // the nid's below correspond to carbon accounts 
  
  $sql = "SELECT  t.*, n.*, u.name, u.uid " .
  		   "FROM {carbon_transfer} t " . 
         "LEFT JOIN {node} n ON t.nid = n.nid " .  
         "INNER JOIN {users} u on n.uid = u.uid "  ;                 
  
  if (empty($account_id))
  {
 		$header = $headerAllAccounts ;   	  	
  }
  else {
  	$header = $headerOneAccount ; 
		$sql .= " WHERE t.seller=%d OR  t.buyer=%d ";
  }
  
  $sql .= tablesort_sql($header); 
  
  // get the links (no range limit here)
  $queryResult = db_query($sql, $account_id, $account_id);
  
  $rows = array(); 
  $balance = 0.0 ; 
  while ($node = db_fetch_object($queryResult)) 
  {
    $access_ro = carbon_access("view", $node, $node->uid);  
    $access_rw = carbon_access("update", $node, $node->uid);
    $link = "node/".$node->nid .($access_rw ? "" : "") ; // go to view mode first

    if ($access_ro)
    {
      $row =  array(_format_date($node->valuedate, "custom", "Ymd"), $node->title);
      
			if (empty($account_id))
      { // show both seller and buyer 
			  $row[] = $account_map->getTransferLink($node->seller);
      	$row[] = $account_map->getTransferLink($node->buyer);
        $row[] = $node->carbon ; 
      	$row[] = $node->cost ; 
      }
      else 
      { // show just the counterparty, debit and credit. 
        if ($node->seller == $account_id)
        {
        	$row[] = $account_map->getTransferLink($node->buyer);
        	$balance += $row[] = $node->cost ; 
          $row[] = "" ;         	
        }
        else  // account is the buyer, cash debit 
        {
        	$row[] = $account_map->getTransferLink($node->seller);
          $row[] = "" ;
        	$balance -= $row[] = $node->cost ; 
        }
      }
 
      if (!empty($account_id))
      {
        $row[] = $balance ; 
      }
      $row[] = $node->name; 
      $row[] = l("view",$link);

      $rows[] = $row ;  
    }
    $sampleRecord = $node ;  
  }
  
  if (empty($rows))
  {
  	$rows[] = array("None viewable");
  }
  $output = theme("table", $header, $rows);
  $output .= theme("pager", NULL, 10, 0);

  $format = $_GET["datatype"]; 
  if ($format == 'json')
  {
  	print($output);
  	return ;  
  }
      
  
  if (empty($account_id))
  {
    $title = t("All Carbon Purchases");   	
  }
	else
	{
    $title = t("Cash Movements for ". $account_map->getTitle($account_id));   			  
	}
    
  drupal_set_title(check_plain($title));      
  return $output;
}



function carbon_transfer_header()
{
  global $carbon_transfer_header ;
  if (!isset($carbon_transfer_header))
  { 	 
    $carbon_stamp_header = array(
    array("data" => t("date"),        "field" => "valuedate", "sort" => "asc"),
    array("data" => t("seller"),      "field" => "seller"),
    array("data" => t("buyer"),       "field" => "buyer"),
	  array("data" => t("carbon"),      "field" => "carbon"),
    array("data" => t("cost"),        "field" => "cost"),
	  array("data" => t("valuedate"),   "field" => "valuedate"));
  }
  return $carbon_transfer_header ; 
}

/**
 * 
 * Joins the recorded stamps in a carbon account with all the available carbon 
 * sources. 
 * 
 * Note that some source fields have been renamed e.g. sourceid, sourceclass, sourcetitle 
 *
 *@param $nid  select stamps attached to this footprint id, can be null
 *@param $uid  select stamps with this uid, can be null  
 *@param $org  select stamps with this org, can be null  
 *@return an array of carbon stamps (and an array of carbon sources) 
 */


function carbon_transfer_array_load($nid, $uid, $org)
{
	$query ="SELECT  ntransfer.title, ntransfer.status, ntransfer.uid, ntransfer.type, " .
    "transfer.* AS shared_uid " .
    "FROM {carbon_transfer} transfer " .
    "INNER JOIN {node} ntransfer ON  transfer.nid = ntransfer.nid ".      		
    "WHERE   ntransfer.status " ;          // transfer must be published     
 
 //   "AND (j.fid = %d  " .                 (empty($nid) ?  " OR 1) " : ") ").
 //   "AND (ntransfer.uid = %d " .          (empty($uid) ?  " OR 1) " : ") ").           
 //   "AND (account.organization = '%s' " . (empty($org) ?  " OR 1) " : ") ").   "" ;   

    if (!empty($nid))
    {
      $query .= "AND  (transfer.seller=%d OR transfer.buyer=%d) ";     
    }
    
//	$query = db_rewrite_sql($query); 
//  drupal_set_message(" my sql ". $query, "info"); 

//  $query .= tablesort_sql(carbon_transfer_header()); 
  $queryResult =  db_query($query, $nid, $nid);   // needs fixing 
	
  $transfers = array();
  $num = 0 ; 
	while ($record = db_fetch_object($queryResult))
	{
    $transfers[] = $record ; 
//    echo $record->title . " bb <br/>" . $query; 
	}   
	return array("transfer" => $transfers);
}


function carbon_transfer_form(&$node, &$param) {
	global $user ; 

  $account_map = new AccountMap();  $account_map->init(); 
  $form = array(); 

    $form["title"] = array(
      "#type" => "textfield",  
      "#title" => "description",
      "#required" => true, 
      "#default_value" => $node->title, 
      "#size" => 10,
      "#maxlength" => 40,    
      "#weight" => 0,
      "#description" => "purpose or reason for transfer");
      
      
    $form["seller_index"] = array(
      "#type" => "select",  
      "#options" => $account_map->getArray(), 
      "#title" => "seller",
      "#required" => true, 
      "#default_value" => $account_map->getArrayIndex($node->seller), 
      "#size" => 10,
      "#maxlength" => 4,    
      "#weight" => 0,
      "#description" => "carbon account to debit");

    $form["buyer_index"] = array(
      "#type" => "select",  
      "#options" =>  $account_map->getArray(), 
      "#title" => "buyer",
      "#required" => true, 
      "#default_value" =>  $account_map->getArrayIndex($node->buyer),
      "#size" => 10,
      "#maxlength" => 4,    
      "#weight" => 1,
      "#description" => "carbon account to credit");

    $form["carbon"] = array(
      "#type" => "textfield",  
      "#title" => "carbon",
      "#required" => true, 
      "#default_value" =>  $node->carbon, 
      "#size" => 20,
      "#maxlength" => 20,    
      "#weight" => 2,
      "#description" => "amount of carbon credit");  // don't specify units'

    $form["cost"] = array(
      "#type" => "textfield",  
      "#title" => "cost",
      "#required" => true, 
      "#default_value" =>  $node->cost, 
      "#size" => 20,
      "#maxlength" => 20,    
      "#weight" => 2,
      "#description" => "monetary value of transaction");     
    
//    $firstdate = mktime(0, 0, 0, substr($params["firstdate"],4,2), substr($params["firstdate"],6,2), substr($params["firstdate"],0,4));  
    
    $form["valuedate_s"] = array(
      "#type" => "textfield",  
      "#title" => "value date",
      "#required" => true, 
      "#default_value" =>  _format_date($node->valuedate == 0 ? time() : $node->valuedate, "custom", "Ymd"),
      "#size" => 20,
      "#maxlength" => 20,    
      "#weight" => 3,
      "#description" => t("Effective date of transfer, format: %time", array("%time" => _format_date(time(), "custom", "Ymd"))));
   
  return $form ;   
}



/**
 * Validate our forms
 * @param string form id
 * @param array form values
 */
function carbon_transfer_validate($form_id, $form_values) 
{
/*
	for ($i = 0 ; $i < 500 ; $i++)
  {
	  _validate_adjustment($form_id, "ff". $i .".adjustment");
    _validate_date($form_id, "ff". $i .".startdate_s"); 
    _validate_date($form_id, "ff". $i .".enddate_s"); 
//    _validate_date($form_id, "ff". $i .".firstdate"); 
  }
*/
}


/**
 * Implementation of hook_load
 */


function carbon_transfer_load(&$node)  {
		
	$query = db_query(
	"SELECT t.* " .
	"FROM {carbon_transfer} t " .
  "LEFT  JOIN {carbon_stamp_account} j ON t.nid = j.sid " .
  "LEFT  JOIN {node}                 n ON j.fid = n.nid " . // to get footprint title
	"WHERE t.nid = %d " .
	" ", $node->nid);
  $first_record = null ; 

  while ($first = db_fetch_object($query))
  {
    return $first ; 
  }
  drupal_set_message("Failed to load carbon_transfer where nid=", $node->nid);
}


function carbon_transfer_submit(&$node)  
{    
  $account_map = new AccountMap();  $account_map->init(); 
  
  $node->seller = $account_map->getNid($node->seller_index);
  $node->buyer = $account_map->getNid($node->buyer_index);
  $node->valuedate = mktime(0, 0, 0, substr($node->valuedate_s,4,2), 
                                     substr($node->valuedate_s,6,2), 
                                     substr($node->valuedate_s,0,4));

//drupal_set_message("ghi " . $node->seller . " ... " . $node->buyer_index) ; 
}

/**
 * Implementation of hook_insert, which saves carbon-specific information
 * into the carbon table
 * @param node object
 */
 
function carbon_transfer_insert(&$node) 
{
	db_query("INSERT INTO {carbon_transfer} ".
	"(nid, seller, buyer, carbon, cost, valuedate) " .
	"VALUES (%d, %d, %d, %f, %f, %d)", 
	$node->nid, $node->seller, $node->buyer, $node->carbon, $node->cost, $node->valuedate); 
}



/**
 * Implementation of hook_update, which saves updated todo-specific 
 * information into the todo table
 * @param node object
 */
function carbon_transfer_update(&$node) 
{
//  drupal_set_message("def " . $node->seller . " ... " . $node->seller_index) ; 
  db_query("UPDATE {carbon_transfer} set seller=%d, buyer=%d, carbon=%f, cost=%f, valuedate=%d " .
    		   " WHERE nid=%d", 
	$node->seller, $node->buyer, $node->carbon, $node->cost, $node->valuedate,
 	$node->nid);
}


/**
 * Implementation of hook_delete()
 */
function carbon_transfer_delete($node) {
	db_query("DELETE FROM {carbon_stamp_account} WHERE sid = %d", $node->nid);   
	db_query("DELETE FROM {carbon_transfer}      WHERE nid = %d", $node->nid);
}


/**
 * Implementation of hook_view, add our node specific information
 * @param node object to display
 * @param boolean is this a teaser or full node?
 * @param boolean is this displaying on its own page
 * 
 */
function carbon_transfer_view(&$node, $teaser = FALSE, $page = FALSE) 
{
  $node = node_prepare($node, $teaser);
  $carbon_account_info = theme('carbon_transfer_table_view', $node); 
  $node->content["transfer"] = array("#value" => $carbon_account_info);   
  $node->body .= $carbon_account_info;
  $node->teaser .= $carbon_account_info;
  return $node ;
}



function theme_carbon_transfer_table_view($node)
{
	// content variable that will be returned for display
  $header = array("valuedate", "description", "seller", "buyer", "carbon", "cost", "published");
  
  $account_map = new AccountMap() ; $account_map->init(); 
  
  $rows = array(); 
  
	$rows[] = array(_format_date($node->valuedate, "custom", "Ymd"),
									$node->title, 
                  $account_map->getAccountLink($node->seller), 
                  $account_map->getAccountLink($node->buyer), 
                  $node->carbon, 
                  $node->cost, 
                  $node->status ? "Yes" : "No");
	
	$output = theme("table", $header, $rows);

  return "<h1></h1>".$output;  // the H1s make bluebreeze theme work
}

/**
 * Implementation of hook_view, add our node specific information
 * @param node object to display
 * @param boolean is this a teaser or full node?
 * @param boolean is this displaying on its own page
 * 
 */
function carbon_transfer_edit(&$node, $teaser = FALSE, $page = FALSE) 
{
  $node = node_prepare($node, $teaser);
  $carbon_account_info = theme('carbon_transfer_table_view', $node);  
  $node->body .= $carbon_account_info;
  $node->teaser .= $carbon_account_info;
}


class CarbonTransaction
{
  var $title ;    // used for references and descriptive text
	var $seller ;  
	var $buyer ; 
	var $carbon ;
	var $cost ; 
	var $valuedate	; 
	
	function initialize()
	{
		global $user ;
    $this->type = 'carbon_transfer' ;
    $this->status = 1 ;  // published
    $this->promote = 0 ;
    $this->sticky = 0;
    $this->body = null ;
    $this->comment = 1 ;
    $this->uid = $user->uid ; 
    $this->teaser = "" ; 
    $this->body = "" ;
	}
	
	function toString($account_map)
	{
    $carbon = sprintf("%.2f", $this->carbon);
    $cost = sprintf("%.2f", $this->cost);
    return $account_map[$this->seller]->title.
		" sold to ".$account_map[$this->buyer]->title.
		", ".$carbon. " carbon credits".
		" at a cost of ".$cost;
	}
}


?>